// includes
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "nrf.h"
#include "nrf_gpio.h"
#include "AAU_uart.h"
#include "boards.h"
#include "nrf_gzll.h"
#include "definitions.h"
#include <stdio.h>


static uint8_t data_payload[RX_PAYLOAD_LENGTH];//NRF_GZLL_CONST_MAX_PAYLOAD_LENGTH];
static uint8_t ack_payload[TX_PAYLOAD_LENGTH];
static uint8_t i = 0;

#ifdef PRESSURE_SENSOR
uint8_t rec_buffer_[PREESSURE_RX_PAYLOAD_LENGTH ] = {0x00, 0x00, 0x00,0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
char trans_buffer_[PREESSURE_RX_PAYLOAD_LENGTH ];
char uart_buf_[24]={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};//'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
#elif defined PROTO13
uint8_t rec_buffer_[RX_PROTO13]; 
char trans_buffer_[PREESSURE_RX_PAYLOAD_LENGTH ];
char uart_buf_[24]={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};//'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
#elif defined GENERAL_CASE
uint8_t rec_buffer_[RX_NI_INTERFACE] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
char trans_buffer_[PREESSURE_RX_PAYLOAD_LENGTH ];
#elif defined LCM_DEMO
uint8_t rec_buffer_[RX_GENERAL]; 
//char trans_buffer_[PREESSURE_RX_PAYLOAD_LENGTH ];
char uart_buf_[24]={'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};//'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};	
#endif



// bool veriables for diagnostics 
static bool init_ok, push_ok, enable_ok, packet_received, pop_ok, tx_success;


// flag set up
bool radio_data_received_ =  false; 
bool radio_data_ready_ = false;

/**
*@brief Main function
*@return int return type requied by ANSI
*/
static uint8_t required_command;
bool data_received_ncap_ = false;

bool processUserCommand()
{
		switch(uart_data[0])
		{
			case 0x30:
				ack_payload[0]=0x00;
			break;
			case 0x31:
				ack_payload[0]=0x01;
			break;
			case 0x32:
				ack_payload[0]=0x02;
			break;
			case 0x33:
				ack_payload[0]=0x03;
			break;
			case 0x34:
				ack_payload[0]=0x04;
			break;
			case 0x35:
				ack_payload[0]=0x05;
			break;
			case 0x36:
				ack_payload[0]=0x06;
			break;
			case 0x37:
				ack_payload[0]=0x07;
			break;
			case 0x38:
				ack_payload[0]=0x08;
			break;
			default:
				break;
		}
}

void fill_uart_buffer()
{
	char *pos = uart_buf_;
		int j = 0;
		for(;j<RX_GENERAL;j++)
	{
		pos += sprintf(pos,"%c",rec_buffer_[j]);
	}		
}

int main(void)
{
	//UART setup
	simple_uart_config(RTS_PIN_NUMBER, TX_PIN_NUMBER, CTS_PIN_NUMBER, RX_PIN_NUMBER, HWFC);
	
	// GPIO setup
	#ifdef LCM_DEMO
	nrf_gpio_pin_dir_set(21,NRF_GPIO_PIN_DIR_OUTPUT); // red color pin set to output for LCM demo
	nrf_gpio_pin_clear(21);
	#else
	nrf_gpio_pin_dir_set(23,NRF_GPIO_PIN_DIR_OUTPUT); // blue color pin set to output 
	nrf_gpio_pin_clear(23);
	#endif
	
	// GEZELL SETUP
	//nrf_gzll_set_timeslot_period(700UL);
	init_ok = nrf_gzll_init(NRF_GZLL_MODE_HOST);
	init_ok = nrf_gzll_set_max_tx_attempts(100);	
	ack_payload[0] = 0x00; //synchronisation to node value 
	//push_ok = nrf_gzll_add_packet_to_tx_fifo(ACTIVE_PIPE, ack_payload, TX_PAYLOAD_LENGTH); // this part will have to be changed to accomodate multiple pipes communication
	enable_ok = nrf_gzll_enable();
	
	while(true)
	{
		if (radio_data_received_ == true)
		{
			//radio_data_received_ = false; // reset flag
			#ifdef STRING_OUTPUT
			uint32_t data_payload_length = NRF_GZLL_CONST_MAX_PAYLOAD_LENGTH;//RX_PAYLOAD_LENGTH;
			fill_uart_buffer();
			simple_uart_put_array(uart_buf_, sizeof(uart_buf_));
			//radio_data_received_ = false;
			#else
			//uint32_t data_payload_length = NRF_GZLL_CONST_MAX_PAYLOAD_LENGTH;//RX_PAYLOAD_LENGTH;
			//simple_uart_put_array(rec_buffer_, sizeof(rec_buffer_));
			//radio_data_received_ = false;
			#endif
			#ifdef LCM_DEMO
			ack_payload[0]=0x00;
			simple_uart_put_array(rec_buffer_, sizeof(rec_buffer_));
			nrf_gzll_add_packet_to_tx_fifo(ACTIVE_PIPE, ack_payload, TX_PAYLOAD_LENGTH);// at this point, the ack_payload should be changed
			radio_data_received_ = false; // reset flag
			#endif

			#ifdef PRESSURE_SENSOR
			nrf_gzll_add_packet_to_tx_fifo(ACTIVE_PIPE, ack_payload, TX_PAYLOAD_LENGTH);
			radio_data_received_ = false; // reset flag
			#elif defined PROTO13
			nrf_gzll_add_packet_to_tx_fifo(ACTIVE_PIPE, ack_payload, TX_PAYLOAD_LENGTH);
			//radio_data_received_ = false; // reset flag	
			#else 
			nrf_gzll_add_packet_to_tx_fifo(PIPE_NUMBER, ack_payload, TX_PAYLOAD_LENGTH);
			//radio_data_received_ = false; // reset flag
			#endif
		}
		/*else if(uart_data_received == true)
		{
			uart_data_received = false;
			#ifdef LCM_DEMO
			processUserCommand(); 
			nrf_gzll_add_packet_to_tx_fifo(ACTIVE_PIPE, ack_payload, TX_PAYLOAD_LENGTH);
			#endif
		}*/
	}
}

void nrf_gzll_host_rx_data_ready(uint32_t pipe, nrf_gzll_host_rx_info_t rx_info)
{
	#ifdef LCM_DEMO
	uint32_t data_payload_length = RX_GENERAL;
	packet_received = true;
	pop_ok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
	memcpy(rec_buffer_,data_payload,RX_GENERAL);
	radio_data_received_ = true;
	#elif defined PRESSURE_SENSOR
	uint32_t data_payload_length = PREESSURE_RX_PAYLOAD_LENGTH;
	packet_received = true;
	pop_ok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
	memcpy(rec_buffer_,data_payload,PREESSURE_RX_PAYLOAD_LENGTH/*26*/);
	radio_data_received_ = true; // set flag
	#elif defined PROTO13
	uint32_t data_payload_length = RX_PROTO13;
	packet_received = true;
	pop_ok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
	memcpy(rec_buffer_,data_payload,RX_PROTO13);
	radio_data_received_ = true; // set flag
	#else
	uint32_t data_payload_length = RX_PAYLOAD_LENGTH;//NRF_GZLL_CONST_MAX_PAYLOAD_LENGTH;
	packet_received = true;
	pop_ok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
	memcpy(rec_buffer_,data_payload,RX_PAYLOAD_LENGTH);
	radio_data_received_ = true; // set flag*/
	#endif
}



// Callbacks not needed in this example.
void nrf_gzll_device_tx_success(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info) {}
void nrf_gzll_device_tx_failed(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info) 
{
	  //tx_success = false;
    //push_ok = nrf_gzll_add_packet_to_tx_fifo(pipe, data_payload, TX_PAYLOAD_LENGTH);
}
void nrf_gzll_disabled() {}